#include "gfx.h"
#include "image.h"
#include "matrix.h"
#include <math.h>
#include <stdio.h>
#include <stdlib.h>

int main() {

  pgm in;
  // pgm tmp;
  pgm out;
  matrix kernel;

  pmg_read_from_file(&in, "chien-pasteque.pgm");
  // https://en.wikipedia.org/wiki/Kernel_(image_processing)

  pmg_photomaton(&out, &in);
  pmg_write_to_file(&out, "photomat.pgm");
  matrix_destroy(&out.pixels);

  pmg_symmetry_hori(&out, &in);
  pmg_write_to_file(&out, "symh.pgm");
  matrix_destroy(&out.pixels);

  pmg_symmetry_vert(&out, &in);
  pmg_write_to_file(&out, "symv.pgm");
  matrix_destroy(&out.pixels);

  pmg_symmetry_cent(&out, &in);
  pmg_write_to_file(&out, "symc.pgm");
  matrix_destroy(&out.pixels);

  pmg_crop(&out, &in, 50, 200, 50, 200);
  pmg_write_to_file(&out, "crop.pgm");
  matrix_destroy(&out.pixels);

  int sharp[] = {0, -1, 0, -1, 5, -1, 0, -1, 0};
  int blur[] = {1, 1, 1, 1, 1, 1, 1, 1, 1};
  int edge1[] = {1, 0, -1, 0, 0, 0, -1, 0, 1};
  int edge2[] = {0, -1, 0, -1, 4, -1, 0, -1, 0};
  int edge3[] = {-1, -1, -1, -1, 8, -1, -1, -1, -1};
  int blur5x5[] = {1,  4, 6, 4,  1,  4,  16, 24, 16, 4, 6, 24, 36, 24, 6, 4, 16, 24, 16, 4,  1,  4,  6, 4, 1};

  matrix_init_from_array(&kernel, 3, 3, sharp);
  pmg_conv(&out, &in, &kernel);
  pmg_write_to_file(&out, "sharp.pgm");
  matrix_destroy(&out.pixels);
  matrix_destroy(&kernel);

  matrix_init_from_array(&kernel, 3, 3, blur);
  pmg_conv(&out, &in, &kernel);
  pmg_write_to_file(&out, "blur.pgm");
  matrix_destroy(&out.pixels);
  matrix_destroy(&kernel);

  matrix_init_from_array(&kernel, 3, 3, edge1);
  pmg_conv(&out, &in, &kernel);
  pmg_write_to_file(&out, "edge1.pgm");
  matrix_destroy(&out.pixels);
  matrix_destroy(&kernel);

  matrix_init_from_array(&kernel, 3, 3, edge2);
  pmg_conv(&out, &in, &kernel);
  pmg_write_to_file(&out, "edge2.pgm");
  matrix_destroy(&out.pixels);
  matrix_destroy(&kernel);

  matrix_init_from_array(&kernel, 3, 3, edge3);
  pmg_conv(&out, &in, &kernel);
  pmg_write_to_file(&out, "edge3.pgm");
  matrix_destroy(&out.pixels);
  matrix_destroy(&kernel);

  matrix_init_from_array(&kernel, 5, 5, blur5x5);
  pmg_conv(&out, &in, &kernel);
  pmg_write_to_file(&out, "blur5x5.pgm");
  matrix_destroy(&out.pixels);
  matrix_destroy(&kernel);

  pmg_negative(&out, &in);
  pmg_write_to_file(&out, "neg.pgm");
  pgm_display(out); //------------>display
  matrix_destroy(&out.pixels);

  matrix_destroy(&in.pixels);

  return EXIT_SUCCESS;
}
