#include "matrix.h"
#include <math.h>
#include <stdio.h>
#include <stdlib.h>

// typedef struct _matrix {
//   int m, n;
//   int **data;
// } matrix;

// m nb ligne n nb col

error_code matrix_alloc(matrix *mat, int m, int n) {

  if (m < 0 || n < 0) {
    return out_of_bounds;
  }
  
  mat->data = malloc(m * sizeof(int *));
  
  if(mat->data == NULL){
        return memory_error;
    }

  for (int i = 0; i < m; i++) {
    mat->data[i] = malloc(n * sizeof(int));
    if(mat->data[i] == NULL){
        return memory_error;
    }
  }

  mat->n = n;
  mat->m = m;

  return ok;
}

error_code matrix_init(matrix *mat, int m, int n, int val) {

if (m < 1 || n < 1) {
    return out_of_bounds;
  }

  matrix_alloc(mat, m, n);

  for (int i = 0; i < m; i++) {   // ligne
    for (int j = 0; j < n; j++) { // col

      mat->data[i][j] = val;
    }
  }
  return ok;
}

error_code matrix_init_from_array(matrix *mat, int m, int n, int data[]) {

if (m < 1 || n < 1) {
    return out_of_bounds;
  }

  mat->data = malloc(m * sizeof(int *));

  for (int i = 0; i < m; i++) {
    mat->data[i] = malloc(n * sizeof(int));
  }

  int k = 0;

  for (int i = 0; i < m; i++) {
    for (int j = 0; j < n; j++) {

      mat->data[i][j] = data[k++];
    }
  }

  mat->n = n;
  mat->m = m;
  return ok;
}

error_code matrix_destroy(matrix *mat) {

  if(mat==NULL){
    printf("pas init");
    return uninitialized;
  }

  for (int i = 0; i < mat->m; i++) {
    free(mat->data[i]);
  }
  free(mat->data);

  mat->n = -1;
  mat->m = -1;

  return ok;
}

error_code matrix_clone(matrix *cloned, const matrix mat) {

  matrix_alloc(cloned, mat.m, mat.n);

  for (int i = 0; i < cloned->m; i++) {   // ligne
    for (int j = 0; j < cloned->n; j++) { // col

      cloned->data[i][j] = mat.data[i][j];
    }
  }
  return ok;
}

error_code matrix_transpose(matrix *transposed, const matrix mat) {

  matrix_alloc(transposed, mat.n, mat.m);

  for (int i = 0; i < transposed->m; i++) {   // ligne
    for (int j = 0; j < transposed->n; j++) { // col

      transposed->data[i][j] = mat.data[j][i];
    }
  }
  return ok;
}

error_code matrix_print(const matrix mat) {

  for (int i = 0; i < mat.m; i++) {
    for (int j = 0; j < mat.n; j++) {

      printf("%d ", mat.data[i][j]);
    }
    printf("\n");
  }
  printf("\n");
  return ok;
}

error_code matrix_extract_submatrix(matrix *sub, const matrix mat, int m0,int m1, int n0, int n1) {

  int mDiff=m1 - m0;
  int nDiff=n1-n0;
  
if (mDiff < 0 || nDiff < 0) {
    return out_of_bounds;
    printf("out of bound");
  }


  sub->m=mDiff;
  sub->n=nDiff;

  matrix_alloc(sub, mDiff, nDiff);

  for (int i = 0; i < mDiff; i++) {   // ligne
    for (int j = 0; j < nDiff; j++) { // col

      sub->data[i][j] = mat.data[m1-sub->m+i][n1-sub->n+j];
    }
  }

  return ok;
}



bool matrix_is_equal(matrix mat1, matrix mat2) {

  if (mat1.m != mat2.m || mat1.n != mat2.n) {
    return false;
  }

  for (int i = 0; i < mat1.m; i++) {   // ligne
    for (int j = 0; j < mat1.n; j++) { // col
      if (mat1.data[i][j] != mat2.data[i][j])
        return false;
    }
  }
  return true;
}

error_code matrix_get(int *elem, const matrix mat, int ix, int iy) {

  if (mat.m < ix || mat.n < iy) {
    return out_of_bounds;
  }
  *elem = mat.data[ix][iy];
  return ok;
}

error_code matrix_set(const matrix mat, int ix, int iy, int elem) {

  if (mat.m < ix || mat.n < iy) {
    return out_of_bounds;
  }
  mat.data[ix][iy] = elem;
  return ok;
}

void mult2(int *elem){
  *elem*=2;
}

void multX(int *elem,int x){
  *elem*=x;
}

error_code matrix_map_ip2(matrix mat,int x, void (*foo)(int *,int)) {

  for (int i = 0; i < mat.m; i++) {   // ligne
    for (int j = 0; j < mat.n; j++) { // col

      foo(&mat.data[i][j],x);
    }
  }
  return ok;
}


error_code matrix_map_ip(matrix mat, void (*foo)(int *)) {

  for (int i = 0; i < mat.m; i++) {   // ligne
    for (int j = 0; j < mat.n; j++) { // col

      foo(&mat.data[i][j]);
    }
  }
  return ok;
}

error_code matrix_map(matrix *mapped, const matrix mat, void (*foo)(int *)) {

matrix_clone(mapped, mat);

for (int i = 0; i < mat.m; i++) {   // ligne
    for (int j = 0; j < mat.n; j++) { // col

      foo(&mapped->data[i][j]);
    }
  }
  return ok;

}