#ifndef _MATRIX_H_
#define _MATRIX_H_

#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>

typedef enum _error_code {
  ok,
  out_of_bounds,
  memory_error,
  uninitialized,
} error_code;

typedef struct _matrix {
  int m, n;
  int **data;
} matrix;

error_code matrix_alloc(matrix *mat, int m, int n);

error_code matrix_init(matrix *mat, int m, int n, int val);

error_code matrix_destroy(matrix *mat);

error_code matrix_init_from_array(matrix *mat, int m, int n, int data[]);

error_code matrix_clone(matrix *cloned, const matrix mat);

error_code matrix_transpose(matrix *transposed, const matrix mat);

error_code matrix_print(const matrix mat);

error_code matrix_extract_submatrix(matrix *sub, const matrix mat, int m0, int m1, int n0, int n1);

bool matrix_is_equal(matrix mat1, matrix mat2);

error_code matrix_get(int *elem, const matrix mat, int ix, int iy);

error_code matrix_set(matrix mat, int ix, int iy, int elem);

void mult2(int *elem);
void multX(int *elem,int x);

error_code matrix_map_ip(matrix mat, void (*foo)(int *));

error_code matrix_map_ip2(matrix mat,int x, void (*foo)(int *,int));

error_code matrix_map(matrix *mapped, const matrix mat, void (*foo)(int *));

#endif
