#include "sort.h"
#include <stdlib.h>

typedef struct _stack {

  int *data;
  int top;
  int capacity;

} stack;

bool isSorted(int size, int tab[size], bool (*comp)(int, int)) {

  for (int i = 1; i < size; i++) {
    if (comp(tab[i - 1], tab[i])) {
    //  printf("not sorted\n");
      return false;
    }
  }
  //printf("sorted !\n");
  return true;
}

void random_tab(int size, int tab[size], int min, int max) {
  assert(max > min);
  for (int i = 0; i < size; i++) {
    tab[i] = min + rand() % (max - min);
  }
}

bool isStackEmpty(stack stack) { return stack.top == -1; }

bool isStackFull(stack stack) { return stack.capacity - 1 == stack.top; }

int stack_top(stack stack) { return stack.data[stack.top]; }

int stack_count(stack stack) { return stack.top + 1; }

void stack_init(stack *stack, int max) {
  stack->data = malloc(sizeof(int) * max);
  stack->top = -1;
  stack->capacity = max;
}

void stack_destroy(stack *stack) {
  free(stack->data);
  stack->top = -1;
  stack->capacity = -1;
}

void stack_add(stack *stack, int item) {

  if (!isStackFull(*stack)) {
    stack->top++;
    stack->data[stack->top] = item;
  }
}

int stack_remove(stack *stack) {
  int tmp;
  tmp = stack->data[stack->top];
  stack->top--;
  return tmp;
}

bool isGreater(int a, int b) { return a > b; }

bool isSmaller(int a, int b) { return a < b; }

void swap(int *a, int *b) {
  int tmp = *a;
  *a = *b;
  *b = tmp;
}

void print_array(int *arr, int size) {
  for (int i = 0; i < size; i++) {
    printf("%d ", arr[i]);
  }
  printf("\n");
}

void transfer(stack *a, stack *b) { stack_add(b, stack_remove(a)); }

void sort(int *sorted, const int *const orig, int nitems, bool (*comp)(int, int)) {
  stack g, d;
  stack_init(&g, nitems);
  stack_init(&d, nitems);
  stack_add(&g, orig[0]);

  for (int i = 1; i < nitems; i++) {

    // transfère les éléments de <pile_g> à <pile_d> // tant que sommet(pile_g)
    // < tab[i]
    while (comp(stack_top(g), orig[i]) && !isStackEmpty(g)) { //stack_top(g) < orig[i]
      transfer(&g, &d);
    }
    while (!comp(stack_top(d) , orig[i]) && !isStackEmpty(d)) { //stack_top(d) >= orig[i]

      transfer(&d, &g);
    }
    // empile l'élément dans <pile_g>
    stack_add(&g, orig[i]);
  }

  while (!isStackEmpty(d)) {
    transfer(&d, &g);
  }

  for (int i = 0; i < nitems; i++) {
    sorted[i] = stack_remove(&g);
  }

  stack_destroy(&d);
  stack_destroy(&g);
}