#include "morse.h"

typedef struct _node {
  char lettre;
  struct _node *child[2]; // 0 pour le point, 1 pour le trait
} node;

void print(arbre tree, int niv) {
  if (tree == NULL) {
    return;
  }
  print(tree->child[0], niv + 1);
  for (int i = 0; i < niv; i++) {
    printf(" -");
  }
  printf(" %c\n", tree->lettre);

  print(tree->child[1], niv + 1);
}

char decoder(char *code, arbre tree) {
  arbre tmp = tree;

  for (int i = 0; i < (int)strlen(code); i++) {
    if (tmp == NULL) {
      return '?';
    }
    if (code[i] != '-' && code[i] != '.') {
      break;
    }
    tmp = tmp->child[code[i] == '-'];
  }
  if (tmp == NULL) {
    return '?';
  }
  return tmp->lettre;
}

void decoder_fichier(char *filename, arbre tree) {
  FILE *fp = fopen(filename, "r");
  assert(NULL != fp);
  int i = 0;
  char code[6];

  while (!feof(fp)) {
    char ch = (char)fgetc(fp);

    switch (ch) {
    case '/':
      printf(" ");
      break;
    case ' ':
      code[i] = '\0';
      printf("%c", decoder(code, tree));
      i = 0;
      break;

    default: // child[0] and child[1]
      code[i] = ch;
      i++;
      break;
    }
  }
  fclose(fp);
}

void encoder_fichier(char *filename, int n, int m, char alphabet_morse[n][m]) {
  FILE *fp = fopen(filename, "r");
  assert(NULL != fp);

  while (!feof(fp)) {
    char ch = (char)fgetc(fp);

    switch (ch) {
    case ' ':
      printf("/");
      break;

    default: // Letters
      for (int i = 0; i < n; i++) {
        if (alphabet_morse[i][0] == ch) {
          for (int j = 1; j < (int)strlen(alphabet_morse[i]); j++) {
            printf("%c", alphabet_morse[i][j]);
          }
        }
      }
      printf(" ");
      break;
    }
  }
  fclose(fp);
}

void placer(char *chaine, arbre *tree) {
  arbre tmp = *tree;
  for (int i = 1; i < (int)strlen(chaine); i++) {
    char ch = chaine[i];
    if (ch != '-' && ch != '.') { // string done
      break;
    }

    if (tmp->child[ch == '-'] == NULL) {
      tmp->child[ch == '-'] = calloc(1, sizeof(node));
    }
    tmp = tmp->child[ch == '-'];
  }
  tmp->lettre = chaine[0];
}

void table_build(char *filename, int n, int m, char alphabet_morse[n][m]) {
  FILE *fp = fopen(filename, "r");
  assert(NULL != fp);

  char line[10];

  for (int i = 0; i < n; i++) {
    fgets(line, 10, fp);

    int j = 0;
    while (line[j] != '\r') {
      alphabet_morse[i][j] = line[j];
      j++;
    }
    alphabet_morse[i][j] = '\0';
  }

  fclose(fp);
}

arbre arbre_build(char *filename) {
  arbre tree = calloc(1, sizeof(node));
  tree->lettre = '?';
  FILE *fp = fopen(filename, "r");

  assert(NULL != fp);
  char chaine[6];
  while (!feof(fp)) {
    fgets(chaine, 6, fp);
    placer(chaine, &tree);
  }
  fclose(fp);
  return tree;
}

void arbre_destroy(arbre tree) {

  if (tree != NULL) {
    if (tree->child[0] != NULL) {
      arbre_destroy(tree->child[0]);
    }
    if (tree->child[1] != NULL) {
      arbre_destroy(tree->child[1]);
    }
    free(tree);
  }
}
