#include "hash.h"

typedef struct _entry_t {
  char *key;
  char *value;
} entry_t;

typedef struct _hm_t {
  element **elements;
  int length;
} hm_t;

int hash(char *str, int length) {
  int val = 0;
  int len = strlen(str);
  for (int i = 0; i < len; ++i) {
    val = 43 * val + str[i];
  }

  return (val % length);
}

// création d'un pointeur vers une hm
hm_t *hm_create(int length) {
  hm_t *hm = malloc(sizeof(hm_t));
  hm->length = length;
  hm->elements = malloc(sizeof(element *) * length);
  for (int i = 0; i < length; ++i) {
    hm->elements[i] = lst_vector_init(); // NULL
  }
  return hm;
}

void destroyEntry(entry_t *entry) {
  free(entry->key);
  free(entry->value);
  free(entry);
}

// destruction de la hm et libération de la mémoire
void hm_destroy(hm_t **hm) {
  for (int i = 0; i < (*hm)->length; ++i) {
    int len = lst_vector_length(&(*hm)->elements[i]);
    for (int j = 0; j < len; j++) {

      element *el = lst_vector_pop(&(*hm)->elements[i]);
      destroyEntry(el->data);
      free(el);
    }
  }
  free((*hm)->elements);
  free(*hm);
  hm = NULL;
}


entry_t *createEntry(char *key, char *value) {
  entry_t *entry = malloc(sizeof(entry_t));
  entry->key = malloc(sizeof(char) * (strlen(key) + 1));
  assert(entry->key!=NULL);
  entry->value = malloc((strlen(value) + 1));
  assert(entry->value!=NULL);
  strcpy(entry->key, key);
  strcpy(entry->value, value);
  return entry;
}

bool isSameKey(char *key, void *x) {
  return 0 == strcmp(((entry_t *)x)->key, key);
}

// insère la paire key-value dans la hm. si key est déjà présente écraser value
// dans la hm.

hm_t *hm_set(hm_t *hm, char *key, char *value) {
  int h = hash(key, hm->length);
  element *el = find_element(&hm->elements[h], isSameKey, key);

  if (el != NULL) { // same key -> update val
  printf("update val\n");
    strcpy(((entry_t *)(el->data))->value, value);
  } else { // key is unique -> push it

    lst_vector_push(&hm->elements[h], (void *)createEntry(key, value));
  }
  
  return hm;
}

// retourne la valeur associé à la clé, key
char *hm_get(hm_t *hm, char *key) {

  element *el =
      find_element(&hm->elements[hash(key, hm->length)], isSameKey, key);
  if (el == NULL) {
    return "NULL";
  }
  
  return ((entry_t *)(el->data))->value;
}

// retire une clé de la hm et la retourne
char* hm_rm(hm_t *hm, char *key){
  element *el =remove_element(&hm->elements[hash(key, hm->length)], isSameKey, key);
  
  if(el==NULL){
    printf("trying to remove non existing entry !\n");
    return "NULL";
  } 

  entry_t *entry=(entry_t *)(el->data);
  char* value=malloc(sizeof(char) * (strlen(entry->value) + 1));  
 
  strcpy(value,entry->value);
  destroyEntry(entry);
  free(el);
  
  return value;
    
}

// affiche le contenu de la hm
void hm_print(hm_t *hm) {
  for (int i = 0; i < hm->length; ++i) {
    int len = lst_vector_length(&hm->elements[i]);
    printf("[%d]: ", i);
    for (int j = 0; j < len; j++) {
      element *el = lst_vector_get(&hm->elements[i], j);
      printf("(%s - %s) ", ((entry_t *)(el->data))->key,
             ((entry_t *)(el->data))->value);
    }
    printf("\n");
  }
}