#include "lst_vec.h"
#include <assert.h>
#include <malloc.h>
#include <stdio.h>
#include <stdlib.h>

// typedef struct _element {
//    void *data;
//    struct _element *next;
// } element;

// typedef element* lst_vector;

void *square(void *a) {
   *(int*)a*=*(int*)a;
  return a;
}

bool isEven(void *a) {
  return *(int*)a%2==0;
}

lst_vector lst_vector_init() {
  return NULL;
}

element* move(lst_vector *v,int index){
  element *current = *v;
  for (int i = 0; i < index; i++) {
    //printf("%d\n",i);
    assert(current != NULL);
    current = current->next;
  }
  return current;
}

int lst_vector_length(lst_vector *v) {

  if (v == NULL) {
    return -1;
  }
  
  int size = 0;
  element *current = *v;
  while (current!= NULL) {
    size++;
    current = current->next;
  }
  return size;
}

lst_vector lst_vector_push(lst_vector *v, void *data) {
  if(v==NULL){
    return NULL;
  }
  element *tmp = malloc(sizeof(element));
  tmp->data = data;
  tmp->next = *v;
  *v = tmp;
  return tmp;
}

element *lst_vector_pop(lst_vector *v) {
  if(v==NULL){
    return NULL;
  }
  element *pop = *v;
  *v = (*v)->next;
  return pop;
}

lst_vector lst_vector_set(lst_vector *v, int index, void *data) {

  if(v==NULL||index<0||index>=lst_vector_length(v)){
    return NULL;
  }
  element *current = move(v, index);
  free(current->data);
  current->data = data;
  return *v;
}

element *lst_vector_get(lst_vector *v, int index) {
  if(v==NULL||index<0||index>=lst_vector_length(v)){
    return NULL;
  }
  element *current = move(v, index);
  return current;
}

element *lst_vector_remove(lst_vector *v, int index) {
  if(v==NULL||index<0||index>=lst_vector_length(v)){
    return NULL;
  }
  element *current = move(v, index-1);
  element *removed = current->next;
  current->next = current->next->next; // jump 1 element
  return removed;
}

lst_vector lst_vector_insert(lst_vector *v, void *data, int index) {

if(v==NULL||index<0||index>lst_vector_length(v)){
    return NULL;
  }

  if(index==0){
    lst_vector_push(v, data);
  }else{
    element *current = move(v, index-1);
    lst_vector_push(&current->next, data);

  }
  return *v;
}

void lst_vector_empty(lst_vector *v) {

  element *delete;

  while ((*v)->next != NULL) {
    delete = *v;
    *v = (*v)->next;
    free(delete);
  }
  free(*v);
}

lst_vector lst_vector_map(lst_vector *v, void *(*f)(void *)) {

  if(v==NULL){
    return NULL;
  }

  int index = 0;
  element *current = *v;
  lst_vector new_vec = lst_vector_init();

  while (current != NULL) {
    lst_vector_insert(&new_vec, f(current->data), index);
    current = current->next;
    index++;
  }
  return new_vec;
}

lst_vector lst_vector_filter(lst_vector *v, bool (*f)(void *)){

  if(v==NULL){
    return NULL;
  }

  int index = 0;
  element *current = *v;
  lst_vector new_vec = lst_vector_init();

  while (current != NULL) {

    if(f(current->data)){
      lst_vector_insert(&new_vec, current->data, index);
      index++;
    }
    current = current->next;
  }
  return new_vec;

}

void vec_print(lst_vector v) {
  while (v != NULL) {
    printf("%d\n", *(int *)(v->data));
    v = v->next;
  }
  printf("\n");
}

void clearLoop(lst_vector v) {
  while (v != NULL) {
    free(v->data);
    v = v->next;
  }
}

int main() {

  lst_vector vec = lst_vector_init();
  for (int i = 0; i < 5; i++) {
    int *x = malloc(sizeof(int));
    *x = i;

    assert(lst_vector_insert(&vec, x, i)!=NULL);
    //lst_vector_push(&vec, x); //same but contrary way !
  }
  printf("size: %d\n\n",lst_vector_length(&vec));
  vec_print(vec);
  lst_vector v2=lst_vector_map(&vec,square);
  assert(v2!=NULL);
  lst_vector v3=lst_vector_filter(&vec, isEven);
  assert(v3!=NULL);
  vec_print(v2);
  vec_print(v3);
   
  clearLoop(vec);
  lst_vector_empty(&vec);
  lst_vector_empty(&v2);
  lst_vector_empty(&v3);

  return EXIT_SUCCESS;
}