#include "PGM.h"
#include <math.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

// typedef struct _pgm {
//     int max;
//     matrix pixels;
// } pgm;

void pgm_create(pgm *p,int max ,matrix mx){
  p->max=max;
  p->pixels=mx;
}

pgm_error pmg_read_from_file(pgm *p, char *filename) {
  assert(NULL != filename && NULL != p);
  

  FILE *f = fopen(filename, "r");

  assert(f != NULL);
  if (f == NULL) {
    printf("error with file read");
    return failure;
  }

  int row;
  int col;
  int max;
  char p5[80];

  fscanf(f, "%s %d %d %d", p5, &row, &col, &max);

  // printf("%s %d %d %d\n", p5, row, col, max);

  matrix_alloc(&(p->pixels), row, col);

  for (int i = 0; i < row; i++) {
    for (int j = 0; j < col; j++) {
      uint8_t tmp;
      fread(&tmp, sizeof(uint8_t), 1, f);
      p->pixels.data[i][j] = (int)tmp;
    }
  }

  fclose(f);
  p->max = max;

  return success;
}

pgm_error pmg_write_to_file(pgm *p, char *filename) {
  assert(NULL != filename && NULL != p);
  char folder[80]="pgm/";
  strcat(folder, filename);
  
  FILE *f = fopen(folder, "w");
  assert(f != NULL);
  if (f == NULL) {
    printf("error with file write");
    return failure;
  }
  fprintf(f, "%s\n%d %d\n%d\n", "P5", p->pixels.m, p->pixels.n, p->max);

  for (int i = 0; i < p->pixels.m; i++) {
    for (int j = 0; j < p->pixels.n; j++) {
      fwrite(&(p->pixels.data[i][j]), sizeof(uint8_t), 1, f);
    }
  }

  fclose(f);

  return success;
}

pgm_error pmg_negative(pgm *neg, const pgm *const orig) {
  if (NULL == orig || NULL == neg) {
    return failure;
  }

  matrix_alloc(&(neg->pixels), orig->pixels.m, orig->pixels.n);
  neg->max = orig->max;

  for (int i = 0; i < orig->pixels.m; i++) {   // ligne
    for (int j = 0; j < orig->pixels.n; j++) { // col
      neg->pixels.data[i][j] = orig->max - orig->pixels.data[i][j];
    }
  }

  return success;
}

pgm_error pmg_symmetry_hori(pgm *sym, const pgm *const orig) {
  if (NULL == orig || NULL == sym) {
    return failure;
  }

  matrix_alloc(&(sym->pixels), orig->pixels.m, orig->pixels.n);
  sym->max = orig->max;

  for (int i = 0; i < orig->pixels.m; i++) {   // ligne
    for (int j = 0; j < orig->pixels.n; j++) { // col

      sym->pixels.data[i][j] = orig->pixels.data[i][orig->pixels.n - 1 - j];
    }
  }

  return success;
}

pgm_error pmg_symmetry_vert(pgm *sym, const pgm *const orig) {
  if (NULL == orig || NULL == sym) {
    return failure;
  }

  matrix_alloc(&(sym->pixels), orig->pixels.m, orig->pixels.n);
  sym->max = orig->max;

  for (int i = 0; i < orig->pixels.m; i++) {   // ligne
    for (int j = 0; j < orig->pixels.n; j++) { // col

      sym->pixels.data[i][j] = orig->pixels.data[orig->pixels.m - 1 - i][j];
    }
  }

  return success;
}

pgm_error pmg_symmetry_cent(pgm *sym, const pgm *const orig) {
  if (NULL == orig || NULL == sym) {
    return failure;
  }

  matrix_alloc(&(sym->pixels), orig->pixels.m, orig->pixels.n);
  sym->max = orig->max;

  for (int i = 0; i < orig->pixels.m; i++) {   // ligne
    for (int j = 0; j < orig->pixels.n; j++) { // col

      sym->pixels.data[i][j] =
          orig->pixels.data[orig->pixels.m - 1 - i][orig->pixels.n - 1 - j];
    }
  }

  return success;
}

pgm_error pmg_photomaton(pgm *photomaton, const pgm *const orig) {
  if (NULL == orig || NULL == photomaton) {
    return failure;
  }

  matrix_alloc(&(photomaton->pixels), orig->pixels.m, orig->pixels.n);
  photomaton->max = orig->max;

  int y;
  int x;

  for (int i = 0; i < orig->pixels.m; i++) {   // ligne
    for (int j = 0; j < orig->pixels.n; j++) { // col

      y = i / 2;
      if (i % 2 != 0) {
        y += orig->pixels.m / 2;
      }

      x = j / 2;
      if (j % 2 != 0) {
        x += orig->pixels.n / 2;
      }

      photomaton->pixels.data[y][x] = orig->pixels.data[i][j];
    }
  }

  return success;
}

// alternative
// int x=0;
// int y=0;
// for (int i = 0; i < orig->pixels.m; i+=2) {   // ligne
//     for (int j = 0; j < orig->pixels.n; j+=2) { // col
//       photomaton->pixels.data[y][x] = orig->pixels.data[i][j];
//       photomaton->pixels.data[y +(orig->pixels.n/2)][x] = orig->pixels.data[i
//       + 1][j]; photomaton->pixels.data[y][x +(orig->pixels.m/2)] =
//       orig->pixels.data[i][j + 1]; photomaton->pixels.data[y
//       +(orig->pixels.n/2)][x +(orig->pixels.m/2)] = orig->pixels.data[i +
//       1][j+ 1];
//     x++;
//     }
//       y++;
// }
// printf("[%d,%d]=[%d,%d]\n", y, x, i, j);
//

pgm_error pmg_crop(pgm *crop, const pgm *const orig, int x0, int x1, int y0,
                   int y1) {

  if (ok !=
      matrix_extract_submatrix(&crop->pixels, orig->pixels, y0, y1, x0, x1)) {
    return failure;
  }

  crop->max = orig->max;
  return success;
}

int get_divider(const matrix *const mat) {
  int divider = 0;
  for (int y = 0; y < mat->m; y++) {
    for (int x = 0; x < mat->n; x++) {
      divider += mat->data[y][x];
    }
  }
  if (divider < 1) {
    divider = 1;
  }
  return divider;
}

pgm_error pmg_conv(pgm *conv, const pgm *const orig, const matrix *const kernel) {
  if (NULL == orig || NULL == conv || NULL == kernel) {
    return failure;
  }
  matrix_alloc(&(conv->pixels), orig->pixels.m, orig->pixels.n);
  conv->max = orig->max;


  int divider = get_divider(kernel);

  for (int i = 0; i < orig->pixels.m; i++) {
    for (int j = 0; j < orig->pixels.n; j++) {
      double pixel = 0.0;

      for (int y = 0; y < kernel->m; y++) {
        for (int x = 0; x < kernel->n; x++) {

          int yi = i - kernel->m / 2 + y;
          int xj = j - kernel->n / 2 + x;
          //if pixel is inside matrix
          if (xj >= 0 && yi >= 0 && xj < orig->pixels.m && yi < orig->pixels.n) {
            pixel += orig->pixels.data[yi][xj] * kernel->data[y][x];
          }
        }
      }
      pixel /= divider;
      //check pixel's val -> 
      if (pixel < 0) {
        pixel = 0;
      } else if (pixel > conv->max) {
        pixel = conv->max;
      }
      //set pixel
      conv->pixels.data[i][j] = (int)pixel;
    }
  }


  return success;
}

