#include "quadTree.h"
#include <stdio.h>

static bool is_leaf(node *nd) { return (NULL == nd->child[0]); }

int max(int x, int y) { return (x >= y ? x : y); }

static int array_max(int size, int tab[size]) {
  int m = INT_MIN;
  for (int i = 0; i < size; i++) {
    m = max(m, tab[i]);
  }
  return m;
}

static int nb_nodes(node *tree) {
  int sum = 1;
  if (!is_leaf(tree)) {
    for (int i = 0; i < CHILDREN; i++) {
      sum += nb_nodes(tree->child[i]);
    }
  }
  return sum;
}

static int sizeOfTree(node *tree) {
  return nb_nodes(tree) * (sizeof(double) + 4 * sizeof(node *));
}

static int tree_max(node *tree) {
  int m = INT_MIN;
  if (!is_leaf(tree)) {
    for (int i = 0; i < CHILDREN; i++) {
      m = max(m, tree->child[i]->ave);
      m = max(m, tree_max(tree->child[i]));
    }
  }
  return m;
}

static int depth(node *a) {
  int p[CHILDREN] = {0};
  if (is_leaf(a)) {
    return 0;
  } else {
    for (int i = 0; i < CHILDREN; i++) {
      p[i] = 1 + depth(a->child[i]);
    }
    return array_max(CHILDREN, p);
  }
}

static void swap(node *a, node *b) {
  node tmp = *a;
  *a = *b;
  *b = tmp;
}

void print(node *tree, int decal, char *sep) {
  if (NULL != tree) {
    for (int i = 0; i < decal; i++) {
      printf("%s", sep);
    }
    printf("%d\n", (int)tree->ave);
    decal++;
    if (!is_leaf(tree)) {
      for (int i = 0; i < CHILDREN; i++) {
        print(tree->child[i], decal, sep);
      }
    }
  }
}

node *tree_create(int depth) {
  node *a = calloc(1, sizeof(node));
  if (depth > 0) {
    for (int i = 0; i < CHILDREN; i++) {
      a->child[i] = tree_create(depth - 1);
    }
  }
  return a;
}

void tree_destroy(node *tree) {
  if (!is_leaf(tree)) {
    for (int i = 0; i < CHILDREN; i++) {
      tree_destroy(tree->child[i]);
    }
  }
  free(tree);
}

void symetry(node *arbre, int bit) { // 0 nada, 1 vertical, 2 hori, 3 center
  bit %= 4;
  if (!is_leaf(arbre)) {
    for (int i = 0; i < CHILDREN; i++) {
      if (i < (i ^ bit)) {
        swap(arbre->child[i], arbre->child[i ^ bit]);
      }
    }
    for (int i = 0; i < CHILDREN; i++) {
      symetry(arbre->child[i], bit);
    }
  }
}

node *position(int li, int col, node *a, int d) {
  node *crt = a;
  do {
    int i = (li >> d) & 1;
    int j = (col >> d) & 1;
    int index = (i << 1) + j;
    crt = crt->child[index];
    d--;
  } while (!is_leaf(crt));
  return crt;
}

void average(node *tree) {
  if (!is_leaf(tree)) {
    tree->ave = 0;
    tree->ave_sq = 0;
    for (int i = 0; i < CHILDREN; i++) {
      average(tree->child[i]);
      tree->ave += tree->child[i]->ave;
      tree->ave_sq += tree->child[i]->ave_sq;
    }
    tree->ave /= CHILDREN;
    tree->ave_sq /= CHILDREN;
  }
}

void matrix2tree(matrix *mat, node *tree) {
  int d = depth(tree) - 1;
  for (int m = 0; m < mat->m; m++) {
    for (int n = 0; n < mat->n; n++) {
      node *crt = position(m, n, tree, d);
      crt->ave = mat->data[m][n];
      crt->ave_sq = (crt->ave) * (crt->ave);
    }
  }
}

void tree2matrix(node *tree, matrix *mat) {
  int d = depth(tree) - 1;
  for (int m = 0; m < mat->m; m++) {
    for (int n = 0; n < mat->n; n++) {
      node *crt = position(m, n, tree, d);
      mat->data[m][n] = crt->ave;
    }
  }
}

node *tree_from_file(char *filename) {
  pgm p;
  pmg_read_from_file(&p, filename);
  node *tree = tree_create(round(log2(p.pixels.m)));

  matrix2tree(&p.pixels, tree);
  average(tree);
  matrix_destroy(&p.pixels);
  return tree;
}

void tree_to_file(node *tree, char *filename) {
  matrix mx;
  int size = pow(2, depth(tree));
  int max = tree_max(tree);

  matrix_alloc(&mx, size, size);
  tree2matrix(tree, &mx);
  pgm out;
  out.max = max;
  out.pixels = mx;
  pmg_write_to_file(&out, filename);
  matrix_destroy(&mx);
}

void compress(node *tree, double seuil) {
  if (!is_leaf(tree)) {
    for (int i = 0; i < CHILDREN; i++) {
      compress(tree->child[i], seuil);
    }

    bool isLastBranch = true;
    for (int i = 0; i < CHILDREN; i++) {
      if (!is_leaf(tree->child[i])) {
        isLastBranch = false;
      }
    }

    if (isLastBranch) {
      double ave2 = tree->ave * tree->ave;
      double var = tree->ave_sq - ave2;
      if (var < seuil * seuil * ave2) { // uniformisé pour utiliser seuil [0,1]
        for (int i = 0; i < CHILDREN; i++) {
          free(tree->child[i]);
          tree->child[i] = NULL;
        }
      }
    }
  }
}

void compress_with_rate(node *tree, double seuil) {
  int nb1 = nb_nodes(tree);
  compress(tree, seuil);
  int nb2 = nb_nodes(tree);
  printf("compresion rate: %d%%\n", nb1 / nb2);
}